@@ -1,5 +1,5 @@
 #!/bin/sh
-srcversion='$MirOS: src/bin/mksh/Build.sh,v 1.484.2.2 2011/07/16 18:03:01 tg Exp $'
+srcversion='$MirOS: src/bin/mksh/Build.sh,v 1.484.2.7 2011/12/11 18:18:18 tg Exp $'
 #-
 # Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
 #	Thorsten Glaser <tg@mirbsd.org>
@@ -22,16 +22,7 @@ srcversion='$MirOS: src/bin/mksh/Build.sh,v 1.484.2.2 2011/07/16 18:03:01 tg Exp
 # People analysing the output must whitelist conftest.c for any kind
 # of compiler warning checks (mirtoconf is by design not quiet).
 #
-# Environment used:	CC CFLAGS CPPFLAGS LDFLAGS LIBS NOWARN NROFF
-#			TARGET_OS TARGET_OSREV
-# Feature selectors:	USE_PRINTF_BUILTIN
-# CPPFLAGS recognised:	MKSH_ASSUME_UTF8 MKSH_BINSHREDUCED MKSH_CLS_STRING
-#			MKSH_CONSERVATIVE_FDS MKSH_MIDNIGHTBSD01ASH_COMPAT
-#			MKSH_NOPWNAM MKSH_NO_LIMITS MKSH_SMALL MKSH_S_NOVI
-#			MKSH_UNEMPLOYED MKSH_DEFAULT_EXECSHELL MKSHRC_PATH
-#			MKSH_DEFAULT_TMPDIR MKSH_CLRTOEOL_STRING MKSH_A4PB
-#			MKSH_NO_DEPRECATED_WARNING MKSH_DONT_EMIT_IDSTRING
-#			MKSH_NOPROSPECTOFWORK MKSH_NO_EXTERNAL_CAT
+# Used environment documentation is at the end of this file.
 
 LC_ALL=C
 export LC_ALL
@@ -206,11 +197,15 @@ EOF
 	test x"$fv" = x"1"
 }
 
+add_cppflags() {
+	CPPFLAGS="$CPPFLAGS $*"
+}
+
 ac_cppflags() {
 	test x"$1" = x"" || fu=$1
 	fv=$2
 	test x"$2" = x"" && eval fv=\$HAVE_$fu
-	CPPFLAGS="$CPPFLAGS -DHAVE_$fu=$fv"
+	add_cppflags -DHAVE_$fu=$fv
 }
 
 ac_test() {
@@ -297,7 +292,7 @@ rmf a.exe* a.out* conftest.c *core lft mksh* no *.bc *.ll *.o \
     Rebuild.sh signames.inc test.sh x vv.out
 
 curdir=`pwd` srcdir=`dirname "$0"` check_categories=
-test -n "$dirname" || dirname=.
+test -n "$srcdir" || srcdir=.
 dstversion=`sed -n '/define MKSH_VERSION/s/^.*"\(.*\)".*$/\1/p' $srcdir/sh.h`
 
 e=echo
@@ -332,7 +327,7 @@ do
 		;;
 	:-g)
 		# checker, debug, valgrind build
-		CPPFLAGS="$CPPFLAGS -DDEBUG"
+		add_cppflags -DDEBUG
 		CFLAGS="$CFLAGS -g3 -fno-builtin"
 		;;
 	:-j)
@@ -393,6 +388,10 @@ else
 fi
 
 test x"$TARGET_OS" = x"" && TARGET_OS=`uname -s 2>/dev/null || uname`
+if test x"$TARGET_OS" = x""; then
+	echo "$me: Set TARGET_OS, your uname is broken!" >&2
+	exit 1
+fi
 oswarn=
 ccpc=-Wc,
 ccpl=-Wl,
@@ -415,7 +414,7 @@ esac
 # Configuration depending on OS name
 case $TARGET_OS in
 AIX)
-	CPPFLAGS="$CPPFLAGS -D_ALL_SOURCE"
+	add_cppflags -D_ALL_SOURCE
 	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
 BeOS)
@@ -435,32 +434,32 @@ FreeBSD)
 	;;
 FreeMiNT)
 	oswarn="; it has minor issues"
-	CPPFLAGS="$CPPFLAGS -D_GNU_SOURCE"
+	add_cppflags -D_GNU_SOURCE
 	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
 GNU)
 	case $CC in
 	*tendracc*) ;;
-	*) CPPFLAGS="$CPPFLAGS -D_GNU_SOURCE" ;;
+	*) add_cppflags -D_GNU_SOURCE ;;
 	esac
 	# define NO_PATH_MAX to use Hurd-only functions
-	CPPFLAGS="$CPPFLAGS -DNO_PATH_MAX"
+	add_cppflags -DNO_PATH_MAX
 	;;
 GNU/kFreeBSD)
 	case $CC in
 	*tendracc*) ;;
-	*) CPPFLAGS="$CPPFLAGS -D_GNU_SOURCE" ;;
+	*) add_cppflags -D_GNU_SOURCE ;;
 	esac
 	;;
 Haiku)
-	CPPFLAGS="$CPPFLAGS -DMKSH_ASSUME_UTF8"
+	add_cppflags -DMKSH_ASSUME_UTF8
 	;;
 HP-UX)
 	;;
 Interix)
 	ccpc='-X '
 	ccpl='-Y '
-	CPPFLAGS="$CPPFLAGS -D_ALL_SOURCE"
+	add_cppflags -D_ALL_SOURCE
 	: ${LIBS='-lcrypt'}
 	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
@@ -470,25 +469,28 @@ IRIX*)
 Linux)
 	case $CC in
 	*tendracc*) ;;
-	*) CPPFLAGS="$CPPFLAGS -D_GNU_SOURCE" ;;
+	*) add_cppflags -D_GNU_SOURCE ;;
 	esac
-	CPPFLAGS="$CPPFLAGS -DSETUID_CAN_FAIL_WITH_EAGAIN"
+	add_cppflags -DSETUID_CAN_FAIL_WITH_EAGAIN
 	: ${HAVE_REVOKE=0}
 	;;
 MidnightBSD)
 	;;
 Minix)
-	CPPFLAGS="$CPPFLAGS -DMKSH_UNEMPLOYED -DMKSH_CONSERVATIVE_FDS"
-	CPPFLAGS="$CPPFLAGS -DMKSH_NO_LIMITS"
-	CPPFLAGS="$CPPFLAGS -D_POSIX_SOURCE -D_POSIX_1_SOURCE=2 -D_MINIX"
+	add_cppflags -DMKSH_UNEMPLOYED
+	add_cppflags -DMKSH_CONSERVATIVE_FDS
+	add_cppflags -DMKSH_NO_LIMITS
+	add_cppflags -D_POSIX_SOURCE -D_POSIX_1_SOURCE=2 -D_MINIX
 	oldish_ed=no-stderr-ed		# /usr/bin/ed(!) is broken
 	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
 MirBSD)
 	;;
 MSYS_*)
-	# probably same as CYGWIN* – need to test; from RT|Chatzilla
-	oswarn='but will probably work'
+	# almost same as CYGWIN* (from RT|Chatzilla)
+	: ${HAVE_SETLOCALE_CTYPE=0}
+	# broken on this OE (from ir0nh34d)
+	: ${HAVE_STDINT_H=0}
 	;;
 NetBSD)
 	;;
@@ -497,15 +499,20 @@ OpenBSD)
 	;;
 OSF1)
 	HAVE_SIG_T=0	# incompatible
-	CPPFLAGS="$CPPFLAGS -D_OSF_SOURCE -D_POSIX_C_SOURCE=200112L"
-	CPPFLAGS="$CPPFLAGS -D_XOPEN_SOURCE=600 -D_XOPEN_SOURCE_EXTENDED"
+	add_cppflags -D_OSF_SOURCE
+	add_cppflags -D_POSIX_C_SOURCE=200112L
+	add_cppflags -D_XOPEN_SOURCE=600
+	add_cppflags -D_XOPEN_SOURCE_EXTENDED
 	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
 Plan9)
-	CPPFLAGS="$CPPFLAGS -D_POSIX_SOURCE -D_LIMITS_EXTENSION"
-	CPPFLAGS="$CPPFLAGS -D_BSD_EXTENSION -D_SUSV2_SOURCE"
+	add_cppflags -D_POSIX_SOURCE
+	add_cppflags -D_LIMITS_EXTENSION
+	add_cppflags -D_BSD_EXTENSION
+	add_cppflags -D_SUSV2_SOURCE
+	add_cppflags -DMKSH_ASSUME_UTF8
 	oswarn=' and will currently not work'
-	CPPFLAGS="$CPPFLAGS -DMKSH_ASSUME_UTF8 -DMKSH_UNEMPLOYED"
+	add_cppflags -DMKSH_UNEMPLOYED
 	;;
 PW32*)
 	HAVE_SIG_T=0	# incompatible
@@ -513,7 +520,7 @@ PW32*)
 	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
 QNX)
-	CPPFLAGS="$CPPFLAGS -D__NO_EXT_QNX"
+	add_cppflags -D__NO_EXT_QNX
 	case $TARGET_OSREV in
 	[012345].*|6.[0123].*|6.4.[01])
 		oldish_ed=no-stderr-ed		# oldish /bin/ed is broken
@@ -522,15 +529,16 @@ QNX)
 	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
 SunOS)
-	CPPFLAGS="$CPPFLAGS -D_BSD_SOURCE -D__EXTENSIONS__"
+	add_cppflags -D_BSD_SOURCE
+	add_cppflags -D__EXTENSIONS__
 	;;
 syllable)
-	CPPFLAGS="$CPPFLAGS -D_GNU_SOURCE"
+	add_cppflags -D_GNU_SOURCE
 	oswarn=' and will currently not work'
 	;;
 ULTRIX)
 	: ${CC=cc -YPOSIX}
-	CPPFLAGS="$CPPFLAGS -Dssize_t=int"
+	add_cppflags -Dssize_t=int
 	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
 UWIN*)
@@ -907,6 +915,7 @@ i=0
 if test $ct = gcc; then
 	# The following tests run with -Werror (gcc only) if possible
 	NOWARN=$DOWARN; phase=u
+	ac_flags 0 wnooverflow -Wno-overflow
 	ac_flags 1 fnostrictaliasing -fno-strict-aliasing
 	ac_flags 1 fstackprotectorall -fstack-protector-all
 	ac_flags 1 fwrapv -fwrapv
@@ -1144,6 +1153,9 @@ ac_ifcpp 'ifdef MKSH_ASSUME_UTF8' isset_MKSH_ASSUME_UTF8 '' \
 ac_ifcpp 'ifdef MKSH_CONSERVATIVE_FDS' isset_MKSH_CONSERVATIVE_FDS '' \
     'if traditional/conservative fd use is requested' && \
     check_categories="$check_categories convfds"
+ac_ifcpp 'ifdef MKSH_DISABLE_DEPRECATED' isset_MKSH_DISABLE_DEPRECATED '' \
+    "if deprecated features are to be omitted" && \
+    check_categories="$check_categories nodeprecated"
 
 #
 # Environment: headers
@@ -1179,7 +1191,7 @@ else
 		#define EXTERN
 		#define MKSH_INCLUDES_ONLY
 		#include "sh.h"
-		__RCSID("$MirOS: src/bin/mksh/Build.sh,v 1.484.2.2 2011/07/16 18:03:01 tg Exp $");
+		__RCSID("$MirOS: src/bin/mksh/Build.sh,v 1.484.2.7 2011/12/11 18:18:18 tg Exp $");
 		int main(void) { printf("Hello, World!\n"); return (0); }
 EOF
 	case $cm in
@@ -1222,10 +1234,11 @@ int off_t_is_large[(LARGE_OFF_T % 2147483629 == 721 &&
 int main(void) { return (0); }' >lft.c
 ac_testn can_lfs '' "for large file support" <lft.c
 save_CPPFLAGS=$CPPFLAGS
-CPPFLAGS="$CPPFLAGS -D_FILE_OFFSET_BITS=64"
+add_cppflags -D_FILE_OFFSET_BITS=64
 ac_testn can_lfs_sus '!' can_lfs 0 "... with -D_FILE_OFFSET_BITS=64" <lft.c
 if test 0 = $HAVE_CAN_LFS_SUS; then
-	CPPFLAGS="$save_CPPFLAGS -D_LARGE_FILES=1"
+	CPPFLAGS=$save_CPPFLAGS
+	add_cppflags -D_LARGE_FILES=1
 	ac_testn can_lfs_aix '!' can_lfs 0 "... with -D_LARGE_FILES=1" <lft.c
 	test 1 = $HAVE_CAN_LFS_AIX || CPPFLAGS=$save_CPPFLAGS
 fi
@@ -1278,7 +1291,7 @@ ac_testn sighandler_t '!' sig_t 0 <<-'EOF'
 	int main(void) { return ((int)(ptrdiff_t)(sighandler_t)(ptrdiff_t)kill(0,0)); }
 EOF
 if test 1 = $HAVE_SIGHANDLER_T; then
-	CPPFLAGS="$CPPFLAGS -Dsig_t=sighandler_t"
+	add_cppflags -Dsig_t=sighandler_t
 	HAVE_SIG_T=1
 fi
 
@@ -1289,11 +1302,11 @@ ac_testn __sighandler_t '!' sig_t 0 <<-'EOF'
 	int main(void) { return ((int)(ptrdiff_t)(__sighandler_t)(ptrdiff_t)kill(0,0)); }
 EOF
 if test 1 = $HAVE___SIGHANDLER_T; then
-	CPPFLAGS="$CPPFLAGS -Dsig_t=__sighandler_t"
+	add_cppflags -Dsig_t=__sighandler_t
 	HAVE_SIG_T=1
 fi
 
-test 1 = $HAVE_SIG_T || CPPFLAGS="$CPPFLAGS -Dsig_t=nosig_t"
+test 1 = $HAVE_SIG_T || add_cppflags -Dsig_t=nosig_t
 ac_cppflags SIG_T
 
 #
@@ -1311,9 +1324,10 @@ for what in name list; do
 		extern const char *const _sys_sig${what}[];
 		int main(void) { return (_sys_sig${what}[0][0]); }
 	EOF
-	if eval "test 1 = \$HAVE__SYS_SIG$uwhat"; then
-		CPPFLAGS="$CPPFLAGS -Dsys_sig$what=_sys_sig$what"
-		eval "HAVE_SYS_SIG$uwhat=1"
+	eval uwhat_v=\$HAVE__SYS_SIG$uwhat
+	if test 1 = "$uwhat_v"; then
+		add_cppflags -Dsys_sig$what=_sys_sig$what
+		eval HAVE_SYS_SIG$uwhat=1
 	fi
 	ac_cppflags SYS_SIG$uwhat
 done
@@ -1492,6 +1506,55 @@ test 1 = $fv || check_categories="$check_categories no-histfile"
 ac_testdone
 ac_cppflags
 
+save_CFLAGS=$CFLAGS
+test 1 = $HAVE_CAN_WNOOVERFLOW && CFLAGS="$CFLAGS -Wno-overflow"
+ac_testn compile_time_asserts_$$ '' 'whether compile-time assertions pass' <<-'EOF'
+	#define MKSH_INCLUDES_ONLY
+	#include "sh.h"
+	struct ctasserts {
+	#define cta(name, assertion) char name[(assertion) ? 1 : -1]
+/* this one should be defined by the standard */
+cta(char_is_1_char, (sizeof(char) == 1) && (sizeof(signed char) == 1) &&
+    (sizeof(unsigned char) == 1));
+/* the next assertion is probably not really needed */
+cta(short_is_2_char, sizeof(short) == 2);
+cta(short_size_no_matter_of_signedness, sizeof(short) == sizeof(unsigned short));
+/* the next assertion is probably not really needed */
+cta(int_is_4_char, sizeof(int) == 4);
+cta(int_size_no_matter_of_signedness, sizeof(int) == sizeof(unsigned int));
+
+cta(long_ge_int, sizeof(long) >= sizeof(int));
+
+/* the next assertion is probably not really needed */
+cta(ari_is_4_char, sizeof(mksh_ari_t) == 4);
+/* but the next three are; we REQUIRE signed integer wraparound */
+cta(ari_is_signed, (mksh_ari_t)-1 < (mksh_ari_t)0);
+cta(ari_has_31_bit, 0 < (mksh_ari_t)(((((mksh_ari_t)1 << 15) << 15) - 1) * 2 + 1));
+cta(ari_sign_32_bit_and_wrap,
+    (mksh_ari_t)(((((mksh_ari_t)1 << 15) << 15) - 1) * 2 + 1) >
+    (mksh_ari_t)(((((mksh_ari_t)1 << 15) << 15) - 1) * 2 + 2));
+/* the next assertion is probably not really needed */
+cta(uari_is_4_char, sizeof(mksh_uari_t) == 4);
+/* but the next four are; we REQUIRE unsigned integer wraparound */
+cta(uari_is_unsigned, (mksh_uari_t)-1 > (mksh_uari_t)0);
+cta(uari_has_31_bit, 0 < (mksh_uari_t)(((((mksh_uari_t)1 << 15) << 15) - 1) * 2 + 1));
+cta(uari_has_32_bit, 0 < (mksh_uari_t)(((((mksh_uari_t)1 << 15) << 15) - 1) * 4 + 3));
+cta(uari_wrap_32_bit,
+    (mksh_uari_t)(((((mksh_uari_t)1 << 15) << 15) - 1) * 4 + 3) >
+    (mksh_uari_t)(((((mksh_uari_t)1 << 15) << 15) - 1) * 4 + 4));
+
+cta(sizet_size_no_matter_of_signedness, sizeof(ssize_t) == sizeof(size_t));
+cta(ptrdifft_sizet_same_size, sizeof(ptrdiff_t) == sizeof(size_t));
+cta(ptrdifft_voidptr_same_size, sizeof(ptrdiff_t) == sizeof(void *));
+cta(ptrdifft_funcptr_same_size, sizeof(ptrdiff_t) == sizeof(void (*)(void)));
+/* our formatting routines assume this */
+cta(ptr_fits_in_long, sizeof(ptrdiff_t) <= sizeof(long));
+	};
+	int main(void) { return (sizeof(struct ctasserts)); }
+EOF
+CFLAGS=$save_CFLAGS
+eval test 1 = \$HAVE_COMPILE_TIME_ASSERTS_$$ || exit 1
+
 #
 # Compiler: Praeprocessor (only if needed)
 #
@@ -1572,7 +1635,7 @@ fi
 
 addsrcs '!' HAVE_STRLCPY strlcpy.c
 addsrcs USE_PRINTF_BUILTIN printf.c
-test 1 = "$USE_PRINTF_BUILTIN" && CPPFLAGS="$CPPFLAGS -DMKSH_PRINTF_BUILTIN"
+test 1 = "$USE_PRINTF_BUILTIN" && add_cppflags -DMKSH_PRINTF_BUILTIN
 test 1 = "$HAVE_CAN_VERB" && CFLAGS="$CFLAGS -verbose"
 
 $e $bi$me: Finished configuration testing, now producing output.$ao
@@ -1591,9 +1654,10 @@ cat >>test.sh <<-EOF
 	pflag='$curdir/mksh'
 	sflag='$srcdir/check.t'
 	usee=0 Pflag=0 uset=0 vflag=0 xflag=0
-	while getopts "C:e:Pp:s:t:v" ch; do case \$ch {
+	while getopts "C:e:fPp:s:t:v" ch; do case \$ch {
 	(C)	check_categories[\${#check_categories[*]}]=\$OPTARG ;;
 	(e)	usee=1; eflag=\$OPTARG ;;
+	(f)	check_categories[\${#check_categories[*]}]=fastbox ;;
 	(P)	Pflag=1 ;;
 	(p)	pflag=\$OPTARG ;;
 	(s)	sflag=\$OPTARG ;;
@@ -1631,10 +1695,22 @@ cat >>test.sh <<-EOF
 	cstr='\$os = defined \$^O ? \$^O : "unknown";'
 	cstr="\$cstr"'print \$os . ", Perl version " . \$];'
 	for perli in \$PERL perl5 perl no; do
-		[[ \$perli = no ]] && exit 1
-		perlos=\$(\$perli -e "\$cstr") 2>/dev/null || continue
-		print "Perl interpreter '\$perli' running on '\$perlos'"
-		[[ -n \$perlos ]] && break
+		if [[ \$perli = no ]]; then
+			print Cannot find a working Perl interpreter, aborting.
+			exit 1
+		fi
+		print "Trying Perl interpreter '\$perli'..."
+		perlos=\$(\$perli -e "\$cstr")
+		rv=\$?
+		print "Errorlevel \$rv, running on '\$perlos'"
+		if (( rv )); then
+			print "=> not using"
+			continue
+		fi
+		if [[ -n \$perlos ]]; then
+			print "=> using it"
+			break
+		fi
 	done
 	exec \$perli "\${args[@]}" "\$@"$tsts
 EOF
@@ -1778,3 +1854,67 @@ $e
 $e Run the regression test suite: ./test.sh
 $e Please also read the sample file dot.mkshrc and the fine manual.
 exit 0
+
+: <<'EOD'
+
+=== Environment used ===
+
+==== build environment ====
+CC				default: cc
+CFLAGS				if empty, defaults to -xO2 or +O2
+				or -O3 -qstrict or -O2, per compiler
+CPPFLAGS			default empty
+LDFLAGS				default empty; added before sources
+LIBS				default empty; added after sources
+				[Interix] default: -lcrypt (XXX still needed?)
+NOWARN				-Wno-error or similar
+NROFF				default: nroff
+TARGET_OS			default: $(uname -s || uname)
+TARGET_OSREV			[QNX] default: $(uname -r)
+
+==== feature selectors ====
+USE_PRINTF_BUILTIN		1 to include (unsupported) printf(1) as builtin
+===== general format =====
+HAVE_STRLEN			ac_test
+HAVE_STRING_H			ac_header
+HAVE_CAN_FSTACKPROTECTORALL	ac_flags
+
+==== cpp definitions ====
+MKSHRC_PATH			"~/.mkshrc" (do not change)
+MKSH_A4PB			force use of arc4random_pushb
+MKSH_ASSUME_UTF8		(0=disabled, 1=enabled; default: unset)
+MKSH_BINSHREDUCED		if */sh or */-sh, enable set -o sh
+MKSH_CLRTOEOL_STRING		"\033[K"
+MKSH_CLS_STRING			"\033[;H\033[J"
+MKSH_CONSERVATIVE_FDS		fd 0-9 for scripts, shell only up to 31
+MKSH_DEFAULT_EXECSHELL		"/bin/sh" (do not change)
+MKSH_DEFAULT_TMPDIR		"/tmp" (do not change)
+MKSH_DISABLE_DEPRECATED		disable code paths scheduled for later removal
+MKSH_DONT_EMIT_IDSTRING		omit RCS IDs from binary
+MKSH_MIDNIGHTBSD01ASH_COMPAT	set -o sh: additional compatibility quirk
+MKSH_NOPROSPECTOFWORK		disable jobs, co-processes, etc. (do not use)
+MKSH_NOPWNAM			skip PAM calls, for -static on eglibc, Solaris
+MKSH_NO_DEPRECATED_WARNING	omit warning when deprecated stuff is run
+MKSH_NO_EXTERNAL_CAT		omit hack to skip cat builtin when flags passed
+MKSH_NO_LIMITS			omit ulimit code
+MKSH_SMALL			omit some code, optimise hard for size (slower)
+MKSH_S_NOVI			disable Vi editing mode (default if MKSH_SMALL)
+MKSH_UNEMPLOYED			disable job control (but not jobs/co-processes)
+
+=== generic installation instructions ===
+
+Set CC and possibly CFLAGS, CPPFLAGS, LDFLAGS, LIBS. If cross-compiling,
+also set TARGET_OS. To disable tests, set e.g. HAVE_STRLCPY=0; to enable
+them, set to a value other than 0 or 1. Ensure /bin/ed is installed. For
+MKSH_SMALL but with Vi mode, add -DMKSH_S_NOVI=0 to CPPFLAGS as well.
+
+Normally, the following command is what you want to run, then:
+$ (sh Build.sh -r -c lto && ./test.sh -v) 2>&1 | tee log
+
+Copy dot.mkshrc to /etc/skel/.mkshrc; install mksh into $prefix/bin; or
+/bin; install the manpage, if omitting the -r flag a catmanpage is made
+using $NROFF. Consider using a forward script as /etc/skel/.mkshrc like
+https://www.mirbsd.org/cvs.cgi/contrib/hosted/tg/deb/mksh/debian/.mkshrc?rev=HEAD
+and put dot.mkshrc as /etc/mkshrc so users need not keep up their HOME.
+
+EOD
