@@ -38,7 +38,7 @@
 #endif
 #endif
 
-__RCSID("$MirOS: src/bin/mksh/funcs.c,v 1.193 2011/07/05 20:12:18 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/funcs.c,v 1.204 2011/12/09 20:40:25 tg Exp $");
 
 #if HAVE_KILLPG
 /*
@@ -53,17 +53,29 @@ __RCSID("$MirOS: src/bin/mksh/funcs.c,v 1.193 2011/07/05 20:12:18 tg Exp $");
 
 /* XXX conditions correct? */
 #if !defined(RLIM_INFINITY) && !defined(MKSH_NO_LIMITS)
-#define MKSH_NO_LIMITS
+#define MKSH_NO_LIMITS	1
 #endif
 
 #ifdef MKSH_NO_LIMITS
-#define c_ulimit c_label
+#define c_ulimit	c_true
 #endif
 
 #if defined(ANDROID)
 static int c_android_lsmod(const char **);
 #endif
 
+static int
+c_true(const char **wp MKSH_A_UNUSED)
+{
+	return (0);
+}
+
+static int
+c_false(const char **wp MKSH_A_UNUSED)
+{
+	return (1);
+}
+
 /*
  * A leading = means assignments before command are kept;
  * a leading * means a POSIX special builtin;
@@ -72,29 +84,29 @@ static int c_android_lsmod(const char **);
  */
 const struct builtin mkshbuiltins[] = {
 	{"*=.", c_dot},
-	{"*=:", c_label},
+	{"*=:", c_true},
 	{"[", c_test},
 	{"*=break", c_brkcont},
-	{T_gbuiltin, c_builtin},
+	{Tgbuiltin, c_builtin},
 	{"*=continue", c_brkcont},
 	{"*=eval", c_eval},
 	{"*=exec", c_exec},
 	{"*=exit", c_exitreturn},
-	{"+false", c_label},
+	{"+false", c_false},
 	{"*=return", c_exitreturn},
-	{T_sgset, c_set},
+	{Tsgset, c_set},
 	{"*=shift", c_shift},
 	{"=times", c_times},
 	{"*=trap", c_trap},
 	{"+=wait", c_wait},
 	{"+read", c_read},
 	{"test", c_test},
-	{"+true", c_label},
+	{"+true", c_true},
 	{"ulimit", c_ulimit},
 	{"+umask", c_umask},
 	{"*=unset", c_unset},
 	/* no =: AT&T manual wrong */
-	{T_palias, c_alias},
+	{Tpalias, c_alias},
 	{"+cd", c_cd},
 	/* dash compatibility hack */
 	{"chdir", c_cd},
@@ -113,8 +125,8 @@ const struct builtin mkshbuiltins[] = {
 #endif
 	{"pwd", c_pwd},
 	{"*=readonly", c_typeset},
-	{T__typeset, c_typeset},
-	{T_punalias, c_unalias},
+	{T_typeset, c_typeset},
+	{Tpunalias, c_unalias},
 	{"whence", c_whence},
 #ifndef MKSH_UNEMPLOYED
 	{"+bg", c_fgbg},
@@ -132,7 +144,7 @@ const struct builtin mkshbuiltins[] = {
 #endif
 #ifdef __MirBSD__
 	/* alias to "true" for historical reasons */
-	{"domainname", c_label},
+	{"domainname", c_true},
 #endif
 #if defined(ANDROID)
 	{"lsmod", c_android_lsmod},
@@ -193,7 +205,6 @@ static const struct t_op b_ops[] = {
 	{"",	TO_NONOP }
 };
 
-static int test_eaccess(const char *, int);
 static int test_oexpr(Test_env *, bool);
 static int test_aexpr(Test_env *, bool);
 static int test_nexpr(Test_env *, bool);
@@ -201,7 +212,7 @@ static int test_primary(Test_env *, bool);
 static Test_op ptest_isa(Test_env *, Test_meta);
 static const char *ptest_getopnd(Test_env *, Test_op, bool);
 static void ptest_error(Test_env *, int, const char *);
-static char *kill_fmt_entry(char *, int, int, const void *);
+static char *kill_fmt_entry(char *, size_t, int, const void *);
 static void p_time(struct shf *, bool, long, int, int,
     const char *, const char *)
     MKSH_A_NONNULL((__nonnull__ (6, 7)));
@@ -232,6 +243,7 @@ c_pwd(const char **wp)
 	}
 	p = current_wd[0] ? (physical ? allocd = do_realpath(current_wd) :
 	    current_wd) : NULL;
+	/* LINTED use of access */
 	if (p && access(p, R_OK) < 0)
 		p = NULL;
 	if (!p && !(p = allocd = ksh_get_wd())) {
@@ -389,8 +401,7 @@ c_print(const char **wp)
 					/* generic function returned Unicode */
 					char ts[4];
 
-					c = utf_wctomb(ts, c - 0x100);
-					ts[c] = 0;
+					ts[utf_wctomb(ts, c - 0x100)] = 0;
 					for (c = 0; ts[c]; ++c)
 						Xput(xs, xp, ts[c]);
 					continue;
@@ -533,9 +544,9 @@ c_whence(const char **wp)
 			if (vflag)
 				shprintf("n %s%s for ",
 				    (tp->flag & EXPORT) ? "exported " : null,
-				    T_alias);
+				    Talias);
 			if (!iam_whence && !vflag)
-				shprintf("%s %s=", T_alias, id);
+				shprintf("%s %s=", Talias, id);
 			print_value_quoted(tp->val.s);
 			break;
 		case CFUNC:
@@ -550,14 +561,14 @@ c_whence(const char **wp)
 						shprintf(" (autoload from %s)",
 						    tp->u.fpath);
 				}
-				shf_puts(T__function, shl_stdout);
+				shf_puts(T_function, shl_stdout);
 			}
 			break;
 		case CSHELL:
 			if (vflag)
 				shprintf("%s %s %s",
 				    (tp->flag & SPEC_BI) ? " special" : null,
-				    "shell", T_builtin);
+				    "shell", Tbuiltin);
 			break;
 		case CTALIAS:
 		case CEXEC:
@@ -568,7 +579,7 @@ c_whence(const char **wp)
 						shprintf("a tracked %s%s for ",
 						    (tp->flag & EXPORT) ?
 						    "exported " : null,
-						    T_alias);
+						    Talias);
 				}
 				shf_puts(tp->val.s, shl_stdout);
 			} else {
@@ -605,9 +616,9 @@ c_typeset(const char **wp)
 	struct block *l;
 	struct tbl *vp, **p;
 	uint32_t fset = 0, fclr = 0, flag;
-	int thing = 0, field, base, optc;
+	int thing = 0, field = 0, base = 0, optc;
 	const char *opts;
-	const char *fieldstr, *basestr;
+	const char *fieldstr = NULL, *basestr = NULL;
 	bool localv = false, func = false, pflag = false, istset = true;
 
 	switch (**wp) {
@@ -638,7 +649,6 @@ c_typeset(const char **wp)
 	/* see comment below regarding possible opions */
 	opts = istset ? "L#R#UZ#afi#lnprtux" : "p";
 
-	fieldstr = basestr = NULL;
 	builtin_opt.flags |= GF_PLUSOPT;
 	/*
 	 * AT&T ksh seems to have 0-9 as options which are multiplied
@@ -714,6 +724,8 @@ c_typeset(const char **wp)
 			flag = EXPORT;
 			break;
 		case '?':
+ errout:
+			set_refflag = SRF_NOP;
 			return (1);
 		}
 		if (builtin_opt.info & GI_PLUS) {
@@ -727,12 +739,12 @@ c_typeset(const char **wp)
 		}
 	}
 
-	field = 0;
 	if (fieldstr && !bi_getn(fieldstr, &field))
-		return (1);
-	base = 0;
-	if (basestr && !bi_getn(basestr, &base))
-		return (1);
+		goto errout;
+	if (basestr && (!bi_getn(basestr, &base) || base < 1 || base > 36)) {
+		bi_errorf("%s: %s", "bad integer base", basestr);
+		goto errout;
+	}
 
 	if (!(builtin_opt.info & GI_MINUSMINUS) && wp[builtin_opt.optind] &&
 	    (wp[builtin_opt.optind][0] == '-' ||
@@ -745,8 +757,7 @@ c_typeset(const char **wp)
 	if (func && (((fset|fclr) & ~(TRACE|UCASEV_AL|EXPORT)) ||
 	    set_refflag != SRF_NOP)) {
 		bi_errorf("only -t, -u and -x options may be used with -f");
-		set_refflag = SRF_NOP;
-		return (1);
+		goto errout;
 	}
 	if (wp[builtin_opt.optind]) {
 		/*
@@ -802,8 +813,7 @@ c_typeset(const char **wp)
 				}
 			} else if (!typeset(wp[i], fset, fclr, field, base)) {
 				bi_errorf("%s: %s", wp[i], "not identifier");
-				set_refflag = SRF_NOP;
-				return (1);
+				goto errout;
 			}
 		}
 		set_refflag = SRF_NOP;
@@ -957,6 +967,7 @@ c_typeset(const char **wp)
 			}
 		}
 	}
+	set_refflag = SRF_NOP;
 	return (0);
 }
 
@@ -1020,12 +1031,12 @@ c_alias(const char **wp)
 	/* "hash -r" means reset all the tracked aliases.. */
 	if (rflag) {
 		static const char *args[] = {
-			T_unalias, "-ta", NULL
+			Tunalias, "-ta", NULL
 		};
 
 		if (!tflag || *wp) {
 			shprintf("%s: -r flag can only be used with -t"
-			    " and without arguments\n", T_alias);
+			    " and without arguments\n", Talias);
 			return (1);
 		}
 		ksh_getopt_reset(&builtin_opt, GF_ERROR);
@@ -1038,7 +1049,7 @@ c_alias(const char **wp)
 		for (p = ktsort(t); (ap = *p++) != NULL; )
 			if ((ap->flag & (ISSET|xflag)) == (ISSET|xflag)) {
 				if (pflag)
-					shprintf("%s ", T_alias);
+					shprintf("%s ", Talias);
 				shf_puts(ap->name, shl_stdout);
 				if (prefix != '+') {
 					shf_putc('=', shl_stdout);
@@ -1063,7 +1074,7 @@ c_alias(const char **wp)
 			ap = ktsearch(t, alias, h);
 			if (ap != NULL && (ap->flag&ISSET)) {
 				if (pflag)
-					shprintf("%s ", T_alias);
+					shprintf("%s ", Talias);
 				shf_puts(ap->name, shl_stdout);
 				if (prefix != '+') {
 					shf_putc('=', shl_stdout);
@@ -1071,7 +1082,7 @@ c_alias(const char **wp)
 				}
 				shf_putc('\n', shl_stdout);
 			} else {
-				shprintf("%s %s %s\n", alias, T_alias,
+				shprintf("%s %s %s\n", alias, Talias,
 				    "not found");
 				rv = 1;
 			}
@@ -1086,7 +1097,9 @@ c_alias(const char **wp)
 				afree(ap->val.s, APERM);
 			}
 			/* ignore values for -t (AT&T ksh does this) */
-			newval = tflag ? search(alias, path, X_OK, NULL) : val;
+			newval = tflag ?
+			    search_path(alias, path, X_OK, NULL) :
+			    val;
 			if (newval) {
 				strdupx(ap->val.s, newval, APERM);
 				ap->flag |= ALLOC|ISSET;
@@ -1246,7 +1259,7 @@ c_fgbg(const char **wp)
 
 /* format a single kill item */
 static char *
-kill_fmt_entry(char *buf, int buflen, int i, const void *arg)
+kill_fmt_entry(char *buf, size_t buflen, int i, const void *arg)
 {
 	const struct kill_info *ki = (const struct kill_info *)arg;
 
@@ -1317,7 +1330,8 @@ c_kill(const char **wp)
 					shprintf("%d\n", n);
 			}
 		} else {
-			int w, j, mess_cols, mess_octs;
+			ssize_t w, mess_cols, mess_octs;
+			int j;
 			struct kill_info ki;
 
 			for (j = NSIG, ki.num_width = 1; j >= 10; j /= 10)
@@ -1522,17 +1536,6 @@ c_bind(const char **wp)
 	return (rv);
 }
 
-/**
- * :, false and true
- * ulimit if MKSH_NO_LIMITS
- * domainname on MirBSD (hysterical raisins, just don't ask)
- */
-int
-c_label(const char **wp)
-{
-	return (wp[0][0] == 'f' ? 1 : 0);
-}
-
 int
 c_shift(const char **wp)
 {
@@ -1710,9 +1713,8 @@ c_dot(const char **wp)
 		bi_errorf("missing argument");
 		return (1);
 	}
-	if ((file = search(cp, path, R_OK, &errcode)) == NULL) {
-		bi_errorf("%s: %s", cp,
-		    errcode ? strerror(errcode) : "not found");
+	if ((file = search_path(cp, path, R_OK, &errcode)) == NULL) {
+		bi_errorf("%s: %s", cp, strerror(errcode));
 		return (1);
 	}
 
@@ -1820,7 +1822,7 @@ c_read(const char **wp)
 #if HAVE_SELECT
 	case 't':
 		if (parse_usec(builtin_opt.optarg, &tv)) {
-			bi_errorf("%s: %s '%s'", T_synerr, strerror(errno),
+			bi_errorf("%s: %s '%s'", Tsynerr, strerror(errno),
 			    builtin_opt.optarg);
 			return (2);
 		}
@@ -1902,7 +1904,7 @@ c_read(const char **wp)
 			rv = 1;
 			goto c_read_out;
 		default:
-			bi_errorf("%s: %s", T_select, strerror(errno));
+			bi_errorf("%s: %s", Tselect, strerror(errno));
 			rv = 2;
 			goto c_read_out;
 		}
@@ -2360,7 +2362,7 @@ c_set(const char **wp)
 	const char **owp;
 
 	if (wp[1] == NULL) {
-		static const char *args[] = { T_set, "-", NULL };
+		static const char *args[] = { Tset, "-", NULL };
 		return (c_typeset(args));
 	}
 
@@ -2695,7 +2697,7 @@ c_mknod(const char **wp)
 #endif
 
 /*-
-   test(1) accepts the following grammar:
+   test(1) roughly accepts the following grammar:
 	oexpr	::= aexpr | aexpr "-o" oexpr ;
 	aexpr	::= nexpr | nexpr "-a" aexpr ;
 	nexpr	::= primary | "!" nexpr ;
@@ -2711,19 +2713,21 @@ c_mknod(const char **wp)
 
 	binary-operator ::= "="|"=="|"!="|"-eq"|"-ne"|"-ge"|"-gt"|"-le"|"-lt"|
 			    "-nt"|"-ot"|"-ef"|
-			    "<"|">"	# rules used for [[ .. ]] expressions
+			    "<"|">"	# rules used for [[ ... ]] expressions
 			    ;
-	operand ::= <any thing>
+	operand ::= <anything>
 */
 
 /* POSIX says > 1 for errors */
-#define T_ERR_EXIT	2
+#define T_ERR_EXIT 2
 
 int
 c_test(const char **wp)
 {
-	int argc, res;
+	int argc, rv, invert = 0;
 	Test_env te;
+	Test_op op;
+	const char *lhs, **swp;
 
 	te.flags = 0;
 	te.isa = ptest_isa;
@@ -2745,63 +2749,88 @@ c_test(const char **wp)
 	te.wp_end = wp + argc;
 
 	/*
-	 * Handle the special cases from POSIX.2, section 4.62.4.
-	 * Implementation of all the rules isn't necessary since
-	 * our parser does the right thing for the omitted steps.
+	 * Attempt to conform to POSIX special cases. This is pretty
+	 * dumb code straight-forward from the 2008 spec, but unless
+	 * the old pdksh code doesn't live from so many assumptions.
+	 * It does, though, inline some calls to '(*te.funcname)()'.
 	 */
-	if (argc <= 5) {
-		const char **owp = wp, **owpend = te.wp_end;
-		int invert = 0;
-		Test_op op;
-		const char *opnd1, *opnd2;
-
-		if (argc >= 2 && ((*te.isa)(&te, TM_OPAREN))) {
-			te.pos.wp = te.wp_end - 1;
-			if ((*te.isa)(&te, TM_CPAREN)) {
-				argc -= 2;
-				te.wp_end--;
-				te.pos.wp = owp + 2;
-			} else {
-				te.pos.wp = owp + 1;
-				te.wp_end = owpend;
-			}
+	switch (argc - 1) {
+	case 0:
+		return (1);
+	case 1:
+ ptest_one:
+		op = TO_STNZE;
+		goto ptest_unary;
+	case 2:
+ ptest_two:
+		if (ptest_isa(&te, TM_NOT)) {
+			++invert;
+			goto ptest_one;
 		}
-
-		while (--argc >= 0) {
-			if ((*te.isa)(&te, TM_END))
-				return (!0);
-			if (argc == 3) {
-				opnd1 = (*te.getopnd)(&te, TO_NONOP, 1);
-				if ((op = (*te.isa)(&te, TM_BINOP))) {
-					opnd2 = (*te.getopnd)(&te, op, 1);
-					res = (*te.eval)(&te, op, opnd1,
-					    opnd2, 1);
-					if (te.flags & TEF_ERROR)
-						return (T_ERR_EXIT);
-					if (invert & 1)
-						res = !res;
-					return (!res);
-				}
-				/* back up to opnd1 */
-				te.pos.wp--;
-			}
-			if (argc == 1) {
-				opnd1 = (*te.getopnd)(&te, TO_NONOP, 1);
-				res = (*te.eval)(&te, TO_STNZE, opnd1,
-				    NULL, 1);
-				if (invert & 1)
-					res = !res;
-				return (!res);
-			}
-			if ((*te.isa)(&te, TM_NOT)) {
-				invert++;
-			} else
-				break;
+		if ((op = ptest_isa(&te, TM_UNOP))) {
+ ptest_unary:
+			rv = test_eval(&te, op, *te.pos.wp++, NULL, true);
+ ptest_out:
+			return ((invert & 1) ? rv : !rv);
 		}
-		te.pos.wp = owp + 1;
-		te.wp_end = owpend;
+		/* let the parser deal with anything else */
+		break;
+	case 3:
+ ptest_three:
+		swp = te.pos.wp;
+		/* use inside knowledge of ptest_getopnd inlined below */
+		lhs = *te.pos.wp++;
+		if ((op = ptest_isa(&te, TM_BINOP))) {
+			/* test lhs op rhs */
+			rv = test_eval(&te, op, lhs, *te.pos.wp++, true);
+			goto ptest_out;
+		}
+		/* back up to lhs */
+		te.pos.wp = swp;
+		if (ptest_isa(&te, TM_NOT)) {
+			++invert;
+			goto ptest_two;
+		}
+		if (ptest_isa(&te, TM_OPAREN)) {
+			swp = te.pos.wp;
+			/* skip operand, without evaluation */
+			te.pos.wp++;
+			/* check for closing parenthesis */
+			op = ptest_isa(&te, TM_CPAREN);
+			/* back up to operand */
+			te.pos.wp = swp;
+			/* if there was a closing paren, handle it */
+			if (op)
+				goto ptest_one;
+			/* backing up is done before calling the parser */
+		}
+		/* let the parser deal with it */
+		break;
+	case 4:
+		if (ptest_isa(&te, TM_NOT)) {
+			++invert;
+			goto ptest_three;
+		}
+		if (ptest_isa(&te, TM_OPAREN)) {
+			swp = te.pos.wp;
+			/* skip two operands, without evaluation */
+			te.pos.wp++;
+			te.pos.wp++;
+			/* check for closing parenthesis */
+			op = ptest_isa(&te, TM_CPAREN);
+			/* back up to first operand */
+			te.pos.wp = swp;
+			/* if there was a closing paren, handle it */
+			if (op)
+				goto ptest_two;
+			/* backing up is done before calling the parser */
+		}
+		/* defer this to the parser */
+		break;
 	}
 
+	/* "The results are unspecified." */
+	te.pos.wp = wp + 1;
 	return (test_parse(&te));
 }
 
@@ -2861,15 +2890,17 @@ test_eval(Test_env *te, Test_op op, const char *opnd1, const char *opnd2,
 
 	/* -r */
 	case TO_FILRD:
-		return (test_eaccess(opnd1, R_OK) == 0);
+		/* LINTED use of access */
+		return (access(opnd1, R_OK) == 0);
 
 	/* -w */
 	case TO_FILWR:
-		return (test_eaccess(opnd1, W_OK) == 0);
+		/* LINTED use of access */
+		return (access(opnd1, W_OK) == 0);
 
 	/* -x */
 	case TO_FILEX:
-		return (test_eaccess(opnd1, X_OK) == 0);
+		return (ksh_access(opnd1, X_OK) == 0);
 
 	/* -a */
 	case TO_FILAXST:
@@ -2964,11 +2995,11 @@ test_eval(Test_env *te, Test_op op, const char *opnd1, const char *opnd2,
 
 	/* -O */
 	case TO_FILUID:
-		return (stat(opnd1, &b1) == 0 && b1.st_uid == ksheuid);
+		return (stat(opnd1, &b1) == 0 && (uid_t)b1.st_uid == ksheuid);
 
 	/* -G */
 	case TO_FILGID:
-		return (stat(opnd1, &b1) == 0 && b1.st_gid == getegid());
+		return (stat(opnd1, &b1) == 0 && (gid_t)b1.st_gid == getegid());
 
 	/*
 	 * Binary Operators
@@ -3066,26 +3097,6 @@ test_eval(Test_env *te, Test_op op, const char *opnd1, const char *opnd2,
 	return (1);
 }
 
-/* On most/all unixen, access() says everything is executable for root... */
-static int
-test_eaccess(const char *pathl, int mode)
-{
-	int rv;
-
-	if ((rv = access(pathl, mode)) == 0 && ksheuid == 0 && (mode & X_OK)) {
-		struct stat statb;
-
-		if (stat(pathl, &statb) < 0)
-			rv = -1;
-		else if (S_ISDIR(statb.st_mode))
-			rv = 0;
-		else
-			rv = (statb.st_mode & (S_IXUSR|S_IXGRP|S_IXOTH)) ?
-			    0 : -1;
-	}
-	return (rv);
-}
-
 int
 test_parse(Test_env *te)
 {
@@ -3515,7 +3526,7 @@ c_rename(const char **wp)
 	if (wp[0] == NULL	/* first argument */ ||
 	    wp[1] == NULL	/* second argument */ ||
 	    wp[2] != NULL	/* no further args please */)
-		bi_errorf(T_synerr);
+		bi_errorf(Tsynerr);
 	else if ((rv = rename(wp[0], wp[1])) != 0) {
 		rv = errno;
 		bi_errorf("%s: %s", "failed", strerror(rv));
@@ -3538,7 +3549,7 @@ c_realpath(const char **wp)
 
 	/* check for exactly one argument */
 	if (wp[0] == NULL || wp[1] != NULL)
-		bi_errorf(T_synerr);
+		bi_errorf(Tsynerr);
 	else if ((buf = do_realpath(wp[0])) == NULL) {
 		rv = errno;
 		bi_errorf("%s: %s", wp[0], strerror(rv));
@@ -3563,7 +3574,7 @@ c_cat(const char **wp)
 #define MKSH_CAT_BUFSIZ 4096
 
 	if ((buf = malloc_osfunc(MKSH_CAT_BUFSIZ)) == NULL) {
-		bi_errorf(T_oomem, (unsigned long)MKSH_CAT_BUFSIZ);
+		bi_errorf(Toomem, (unsigned long)MKSH_CAT_BUFSIZ);
 		return (1);
 	}
 
@@ -3574,7 +3585,7 @@ c_cat(const char **wp)
 			/* we already operate unbuffered */
 			break;
 		default:
-			bi_errorf(T_synerr);
+			bi_errorf(Tsynerr);
 			return (1);
 		}
 	}
@@ -3652,9 +3663,9 @@ c_sleep(const char **wp)
 		++wp;
 
 	if (!wp[0] || wp[1])
-		bi_errorf(T_synerr);
+		bi_errorf(Tsynerr);
 	else if (parse_usec(wp[0], &tv))
-		bi_errorf("%s: %s '%s'", T_synerr, strerror(errno), wp[0]);
+		bi_errorf("%s: %s '%s'", Tsynerr, strerror(errno), wp[0]);
 	else {
 #ifndef MKSH_NOPROSPECTOFWORK
 		sigset_t omask;
@@ -3662,14 +3673,14 @@ c_sleep(const char **wp)
 		/* block SIGCHLD from interrupting us, though */
 		sigprocmask(SIG_BLOCK, &sm_sigchld, &omask);
 #endif
-		if (select(0, NULL, NULL, NULL, &tv) == 0 || errno == EINTR)
+		if (select(1, NULL, NULL, NULL, &tv) == 0 || errno == EINTR)
 			/*
 			 * strictly speaking only for SIGALRM, but the
 			 * execution may be interrupted by other signals
 			 */
 			rv = 0;
 		else
-			bi_errorf("%s: %s", T_select, strerror(errno));
+			bi_errorf("%s: %s", Tselect, strerror(errno));
 #ifndef MKSH_NOPROSPECTOFWORK
 		sigprocmask(SIG_SETMASK, &omask, NULL);
 #endif
