@@ -26,15 +26,7 @@
 #include <sys/file.h>
 #endif
 
-__RCSID("$MirOS: src/bin/mksh/histrap.c,v 1.109 2011/04/22 12:21:53 tg Exp $");
-
-/*-
- * MirOS: This is the default mapping type, and need not be specified.
- * IRIX doesn't have this constant.
- */
-#ifndef MAP_FILE
-#define MAP_FILE	0
-#endif
+__RCSID("$MirOS: src/bin/mksh/histrap.c,v 1.113 2011/12/10 14:12:16 tg Exp $");
 
 Trap sigtraps[NSIG + 1];
 static struct sigaction Sigact_ign;
@@ -60,14 +52,31 @@ static int hstarted;		/* set after hist_init() called */
 static Source *hist_source;
 
 #if HAVE_PERSISTENT_HISTORY
+/*XXX imake style */
+#if defined(__linux)
+#define caddr_cast(x)	((void *)(x))
+#else
+#define caddr_cast(x)	((caddr_t)(x))
+#endif
+
+/* several OEs do not have these constants */
+#ifndef MAP_FAILED
+#define MAP_FAILED	caddr_cast(-1)
+#endif
+
+/* some OEs need the default mapping type specified */
+#ifndef MAP_FILE
+#define MAP_FILE	0
+#endif
+
 /* current history file: name, fd, size */
 static char *hname;
 static int histfd;
-static int hsize;
+static size_t hsize;
 #endif
 
-static const char T_not_in_history[] = "not in history";
-#define T_history (T_not_in_history + 7)
+static const char Tnot_in_history[] = "not in history";
+#define Thistory (Tnot_in_history + 7)
 
 int
 c_fc(const char **wp)
@@ -83,7 +92,7 @@ c_fc(const char **wp)
 	char **hfirst, **hlast, **hp;
 
 	if (!Flag(FTALKING_I)) {
-		bi_errorf("history %ss not available", T_function);
+		bi_errorf("history %ss not available", Tfunction);
 		return (1);
 	}
 
@@ -290,7 +299,7 @@ c_fc(const char **wp)
 		if (stat(tf->name, &statb) < 0)
 			n = 128;
 		else if (statb.st_size > (1024 * 1048576)) {
-			bi_errorf("%s %s too large: %lu", T_history,
+			bi_errorf("%s %s too large: %lu", Thistory,
 			    "file", (unsigned long)statb.st_size);
 			goto errout;
 		} else
@@ -365,9 +374,9 @@ hist_replace(char **hp, const char *pat, const char *rep, bool globr)
 		strdupx(line, *hp, ATEMP);
 	else {
 		char *s, *s1;
-		int pat_len = strlen(pat);
-		int rep_len = strlen(rep);
-		int len;
+		size_t pat_len = strlen(pat);
+		size_t rep_len = strlen(rep);
+		size_t len;
 		XString xs;
 		char *xp;
 		bool any_subst = false;
@@ -414,14 +423,14 @@ hist_get(const char *str, bool approx, bool allow_cur)
 			if (approx)
 				hp = hist_get_oldest();
 			else {
-				bi_errorf("%s: %s", str, T_not_in_history);
+				bi_errorf("%s: %s", str, Tnot_in_history);
 				hp = NULL;
 			}
 		} else if ((ptrdiff_t)hp > (ptrdiff_t)histptr) {
 			if (approx)
 				hp = hist_get_newest(allow_cur);
 			else {
-				bi_errorf("%s: %s", str, T_not_in_history);
+				bi_errorf("%s: %s", str, Tnot_in_history);
 				hp = NULL;
 			}
 		} else if (!allow_cur && hp == histptr) {
@@ -433,7 +442,7 @@ hist_get(const char *str, bool approx, bool allow_cur)
 
 		/* the -1 is to avoid the current fc command */
 		if ((n = findhist(histptr - history - 1, 0, str, anchored)) < 0)
-			bi_errorf("%s: %s", str, T_not_in_history);
+			bi_errorf("%s: %s", str, Tnot_in_history);
 		else
 			hp = &history[n];
 	}
@@ -509,10 +518,10 @@ histnum(int n)
 int
 findhist(int start, int fwd, const char *str, int anchored)
 {
-	char	**hp;
-	int	maxhist = histptr - history;
-	int	incr = fwd ? 1 : -1;
-	int	len = strlen(str);
+	char **hp;
+	int maxhist = histptr - history;
+	int incr = fwd ? 1 : -1;
+	size_t len = strlen(str);
 
 	if (start < 0 || start >= maxhist)
 		start = maxhist;
@@ -527,8 +536,7 @@ findhist(int start, int fwd, const char *str, int anchored)
 }
 
 /*
- *	set history
- *	this means reallocating the dataspace
+ * set history; this means reallocating the dataspace
  */
 void
 sethistsize(int n)
@@ -551,9 +559,8 @@ sethistsize(int n)
 
 #if HAVE_PERSISTENT_HISTORY
 /*
- *	set history file
- *	This can mean reloading/resetting/starting history file
- *	maintenance
+ * set history file; this can mean reloading/resetting/starting
+ * history file maintenance
  */
 void
 sethistfile(const char *name)
@@ -567,7 +574,7 @@ sethistfile(const char *name)
 		return;
 
 	/*
-	 * its a new name - possibly
+	 * it's a new name - possibly
 	 */
 	if (histfd) {
 		/* yes the file is open */
@@ -586,7 +593,7 @@ sethistfile(const char *name)
 #endif
 
 /*
- *	initialise the history vector
+ * initialise the history vector
  */
 void
 init_histvec(void)
@@ -600,11 +607,7 @@ init_histvec(void)
 
 
 /*
- *	Routines added by Peter Collinson BSDI(Europe)/Hillside Systems to
- *	a) permit HISTSIZE to control number of lines of history stored
- *	b) maintain a physical history file
- *
- *	It turns out that there is a lot of ghastly hackery here
+ * It turns out that there is a lot of ghastly hackery here
  */
 
 #if !defined(MKSH_SMALL) && HAVE_PERSISTENT_HISTORY
@@ -670,25 +673,23 @@ histsave(int *lnp, const char *cmd, bool dowrite MKSH_A_UNUSED, bool ignoredups)
 }
 
 /*
- *	Write history data to a file nominated by HISTFILE
- *	if HISTFILE is unset then history still happens, but
- *	the data is not written to a file
- *	All copies of ksh looking at the file will maintain the
- *	same history. This is ksh behaviour.
+ * Write history data to a file nominated by HISTFILE;
+ * if HISTFILE is unset then history still happens, but
+ * the data is not written to a file. All copies of ksh
+ * looking at the file will maintain the same history.
+ * This is ksh behaviour.
  *
- *	This stuff uses mmap()
- *	if your system ain't got it - then you'll have to undef HISTORYFILE
+ * This stuff uses mmap()
  */
 
 /*-
- *	Open a history file
- *	Format is:
- *	Bytes 1, 2:
- *		HMAGIC - just to check that we are dealing with
- *		the correct object
- *	Then follows a number of stored commands
- *	Each command is
- *	<command byte><command number(4 bytes)><bytes><null>
+ * Open a history file
+ * Format is:
+ * Bytes 1, 2:
+ *	HMAGIC - just to check that we are dealing with the correct object
+ * Then follows a number of stored commands
+ * Each command is
+ *	<command byte><command number(4 bytes)><bytes><NUL>
  */
 #define HMAGIC1		0xab
 #define HMAGIC2		0xcd
@@ -700,6 +701,7 @@ hist_init(Source *s)
 #if HAVE_PERSISTENT_HISTORY
 	unsigned char *base;
 	int lines, fd, rv = 0;
+	off_t hfsize;
 #endif
 
 	if (Flag(FTALKING) == 0)
@@ -725,7 +727,10 @@ hist_init(Source *s)
 
 	(void)flock(histfd, LOCK_EX);
 
-	hsize = lseek(histfd, (off_t)0, SEEK_END);
+	hfsize = lseek(histfd, (off_t)0, SEEK_END);
+	hsize = 1024 * 1048576;
+	if (hfsize < (off_t)hsize)
+		hsize = (size_t)hfsize;
 
 	if (hsize == 0) {
 		/* add magic */
@@ -745,7 +750,7 @@ hist_init(Source *s)
 		if (base == (unsigned char *)MAP_FAILED ||
 		    *base != HMAGIC1 || base[1] != HMAGIC2) {
 			if (base != (unsigned char *)MAP_FAILED)
-				munmap((caddr_t)base, hsize);
+				munmap(caddr_cast(base), hsize);
 			hist_finish();
 			if (unlink(hname) /* fails */)
 				goto hiniterr;
@@ -757,7 +762,7 @@ hist_init(Source *s)
 				/* we need to make the file smaller */
 				if (hist_shrink(base, hsize))
 					rv = unlink(hname);
-				munmap((caddr_t)base, hsize);
+				munmap(caddr_cast(base), hsize);
 				hist_finish();
 				if (rv) {
  hiniterr:
@@ -771,19 +776,24 @@ hist_init(Source *s)
 			}
 		}
 		histload(hist_source, base+2, hsize-2);
-		munmap((caddr_t)base, hsize);
+		munmap(caddr_cast(base), hsize);
 	}
 	(void)flock(histfd, LOCK_UN);
-	hsize = lseek(histfd, (off_t)0, SEEK_END);
+	hfsize = lseek(histfd, (off_t)0, SEEK_END);
+	hsize = 1024 * 1048576;
+	if (hfsize < (off_t)hsize)
+		hsize = hfsize;
 #endif
 }
 
 #if HAVE_PERSISTENT_HISTORY
 typedef enum state {
-	shdr,		/* expecting a header */
-	sline,		/* looking for a null byte to end the line */
-	sn1,		/* bytes 1 to 4 of a line no */
-	sn2, sn3, sn4
+	/* expecting a header */
+	shdr,
+	/* looking for a NUL byte to end the line */
+	sline,
+	/* bytes 1 to 4 of a line number */
+	sn1, sn2, sn3, sn4
 } State;
 
 static int
@@ -818,7 +828,7 @@ hist_count_lines(unsigned char *base, int bytes)
 }
 
 /*
- *	Shrink the history file to histsize lines
+ * Shrink the history file to HISTSIZE lines
  */
 static int
 hist_shrink(unsigned char *oldbase, int oldbytes)
@@ -836,7 +846,7 @@ hist_shrink(unsigned char *oldbase, int oldbytes)
 		return (0);
 
 	/*
-	 *	create temp file
+	 * create temp file
 	 */
 	nfile = shf_smprintf("%s.%d", hname, (int)procpid);
 	if ((fd = open(nfile, O_CREAT | O_TRUNC | O_WRONLY, 0600)) < 0)
@@ -851,7 +861,7 @@ hist_shrink(unsigned char *oldbase, int oldbytes)
 	fd = -1;
 
 	/*
-	 *	rename
+	 * rename
 	 */
 	if (rename(nfile, hname) < 0) {
  errout:
@@ -867,8 +877,8 @@ hist_shrink(unsigned char *oldbase, int oldbytes)
 }
 
 /*
- *	find a pointer to the data 'no' back from the end of the file
- *	return the pointer and the number of bytes left
+ * find a pointer to the data 'no' back from the end of the file;
+ * return the pointer and the number of bytes left
  */
 static unsigned char *
 hist_skip_back(unsigned char *base, int *bytes, int no)
@@ -880,7 +890,7 @@ hist_skip_back(unsigned char *base, int *bytes, int no)
 		/*
 		 * this doesn't really work: the 4 byte line number that
 		 * is encoded after the COMMAND byte can itself contain
-		 * the COMMAND byte....
+		 * the COMMAND byte...
 		 */
 		for (; ep > base && *ep != COMMAND; ep--)
 			;
@@ -895,7 +905,7 @@ hist_skip_back(unsigned char *base, int *bytes, int no)
 }
 
 /*
- *	load the history structure from the stored data
+ * load the history structure from the stored data
  */
 static void
 histload(Source *s, unsigned char *base, int bytes)
@@ -911,27 +921,27 @@ histload(Source *s, unsigned char *base, int bytes)
 				state = sn1;
 			break;
 		case sn1:
-			lno = (((*base)&0xff)<<24);
+			lno = (((*base) & 0xFF) << 24);
 			state = sn2;
 			break;
 		case sn2:
-			lno |= (((*base)&0xff)<<16);
+			lno |= (((*base) & 0xFF) << 16);
 			state = sn3;
 			break;
 		case sn3:
-			lno |= (((*base)&0xff)<<8);
+			lno |= (((*base) & 0xFF) << 8);
 			state = sn4;
 			break;
 		case sn4:
-			lno |= (*base)&0xff;
-			line = base+1;
+			lno |= (*base) & 0xFF;
+			line = base + 1;
 			state = sline;
 			break;
 		case sline:
 			if (*base == '\0') {
 				/* worry about line numbers */
 				if (histptr >= history && lno-1 != s->line) {
-					/* a replacement ? */
+					/* a replacement? */
 					histinsert(s, lno, (char *)line);
 				} else {
 					s->line = lno--;
@@ -945,7 +955,7 @@ histload(Source *s, unsigned char *base, int bytes)
 }
 
 /*
- *	Insert a line into the history at a specified number
+ * Insert a line into the history at a specified number
  */
 static void
 histinsert(Source *s, int lno, const char *line)
@@ -961,45 +971,44 @@ histinsert(Source *s, int lno, const char *line)
 }
 
 /*
- *	write a command to the end of the history file
- *	This *MAY* seem easy but it's also necessary to check
- *	that the history file has not changed in size.
- *	If it has - then some other shell has written to it
- *	and we should read those commands to update our history
+ * write a command to the end of the history file
+ *
+ * This *MAY* seem easy but it's also necessary to check
+ * that the history file has not changed in size.
+ * If it has - then some other shell has written to it and
+ * we should (re)read those commands to update our history
  */
 static void
 writehistfile(int lno, char *cmd)
 {
-	int	sizenow;
-	unsigned char	*base;
-	unsigned char	*news;
-	int	bytes;
-	unsigned char	hdr[5];
+	off_t sizenow;
+	ssize_t bytes;
+	unsigned char *base, *news, hdr[5];
 
 	(void)flock(histfd, LOCK_EX);
 	sizenow = lseek(histfd, (off_t)0, SEEK_END);
-	if (sizenow != hsize) {
+	if ((sizenow <= (1024 * 1048576)) && ((size_t)sizenow != hsize)) {
 		/*
-		 *	Things have changed
+		 * Things have changed
 		 */
-		if (sizenow > hsize) {
+		if ((size_t)sizenow > hsize) {
 			/* someone has added some lines */
-			bytes = sizenow - hsize;
-			base = (void *)mmap(NULL, sizenow, PROT_READ,
+			bytes = (size_t)sizenow - hsize;
+			base = (void *)mmap(NULL, (size_t)sizenow, PROT_READ,
 			    MAP_FILE | MAP_PRIVATE, histfd, (off_t)0);
 			if (base == (unsigned char *)MAP_FAILED)
 				goto bad;
 			news = base + hsize;
 			if (*news != COMMAND) {
-				munmap((caddr_t)base, sizenow);
+				munmap(caddr_cast(base), (size_t)sizenow);
 				goto bad;
 			}
 			hist_source->line--;
 			histload(hist_source, news, bytes);
 			hist_source->line++;
 			lno = hist_source->line;
-			munmap((caddr_t)base, sizenow);
-			hsize = sizenow;
+			munmap(caddr_cast(base), (size_t)sizenow);
+			hsize = (size_t)sizenow;
 		} else {
 			/* it has shrunk */
 			/* but to what? */
@@ -1009,18 +1018,21 @@ writehistfile(int lno, char *cmd)
 	}
 	if (cmd) {
 		/*
-		 *	we can write our bit now
+		 * we can write our bit now
 		 */
 		hdr[0] = COMMAND;
-		hdr[1] = (lno>>24)&0xff;
-		hdr[2] = (lno>>16)&0xff;
-		hdr[3] = (lno>>8)&0xff;
-		hdr[4] = lno&0xff;
+		hdr[1] = (lno >> 24) & 0xFF;
+		hdr[2] = (lno >> 16) & 0xFF;
+		hdr[3] = (lno >> 8) & 0xFF;
+		hdr[4] = lno & 0xFF;
 		bytes = strlen(cmd) + 1;
 		if ((write(histfd, hdr, 5) != 5) ||
 		    (write(histfd, cmd, bytes) != bytes))
 			goto bad;
-		hsize = lseek(histfd, (off_t)0, SEEK_END);
+		sizenow = lseek(histfd, (off_t)0, SEEK_END);
+		hsize = 1024 * 1048576;
+		if (sizenow < (off_t)hsize)
+			hsize = (size_t)sizenow;
 	}
 	(void)flock(histfd, LOCK_UN);
 	return;
@@ -1037,7 +1049,7 @@ hist_finish(void)
 }
 
 /*
- *	add magic to the history file
+ * add magic to the history file
  */
 static int
 sprinkle(int fd)
@@ -1069,7 +1081,7 @@ inittraps(void)
 	/* Populate sigtraps based on sys_signame and sys_siglist. */
 	for (i = 0; i <= NSIG; i++) {
 		sigtraps[i].signal = i;
-		if (i == SIGERR_) {
+		if (i == ksh_SIGERR) {
 			sigtraps[i].name = "ERR";
 			sigtraps[i].mess = "Error handler";
 		} else {
@@ -1108,7 +1120,7 @@ inittraps(void)
 		}
 	}
 	/* our name for signal 0 */
-	sigtraps[SIGEXIT_].name = "EXIT";
+	sigtraps[ksh_SIGEXIT].name = "EXIT";
 
 	(void)sigemptyset(&Sigact_ign.sa_mask);
 	Sigact_ign.sa_flags = 0; /* interruptible */
@@ -1186,7 +1198,7 @@ void
 trapsig(int i)
 {
 	Trap *p = &sigtraps[i];
-	int errno_ = errno;
+	int errno_sv = errno;
 
 	trap = p->set = 1;
 	if (p->flags & TF_DFL_INTR)
@@ -1197,7 +1209,7 @@ trapsig(int i)
 	}
 	if (p->shtrap)
 		(*p->shtrap)(i);
-	errno = errno_;
+	errno = errno_sv;
 }
 
 /*
@@ -1311,7 +1323,7 @@ runtrap(Trap *p, bool is_last)
 	if (trapstr[0] == '\0')
 		/* SIG_IGN */
 		goto donetrap;
-	if (i == SIGEXIT_ || i == SIGERR_) {
+	if (i == ksh_SIGEXIT || i == ksh_SIGERR) {
 		/* avoid recursion on these */
 		old_changed = p->flags & TF_CHANGED;
 		p->flags &= ~TF_CHANGED;
@@ -1324,7 +1336,7 @@ runtrap(Trap *p, bool is_last)
 	 * no problem with afree(p->trap) in settrap() while still in use.
 	 */
 	command(trapstr, current_lineno);
-	if (i == SIGEXIT_ || i == SIGERR_) {
+	if (i == ksh_SIGEXIT || i == ksh_SIGERR) {
 		if (p->flags & TF_CHANGED)
 			/* don't clear TF_CHANGED */
 			afree(trapstr, APERM);
@@ -1446,7 +1458,7 @@ setsig(Trap *p, sig_t f, int flags)
 {
 	struct sigaction sigact;
 
-	if (p->signal == SIGEXIT_ || p->signal == SIGERR_)
+	if (p->signal == ksh_SIGEXIT || p->signal == ksh_SIGERR)
 		return (1);
 
 	/*
