@@ -26,7 +26,7 @@
 #include <sys/sysctl.h>
 #endif
 
-__RCSID("$MirOS: src/bin/mksh/var.c,v 1.130.2.1 2011/07/07 21:42:20 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/var.c,v 1.130.2.6 2011/12/11 18:18:30 tg Exp $");
 
 /*-
  * Variables
@@ -170,7 +170,7 @@ static const char *
 array_index_calc(const char *n, bool *arrayp, uint32_t *valp)
 {
 	const char *p;
-	int len;
+	size_t len;
 	char *ap = NULL;
 
 	*arrayp = false;
@@ -469,10 +469,10 @@ setint(struct tbl *vq, mksh_ari_t n)
 static int
 getint(struct tbl *vp, mksh_ari_t *nump, bool arith)
 {
-	char *s;
 	int c, base, neg;
+	mksh_uari_t num;
+	const char *s;
 	bool have_base = false;
-	mksh_ari_t num;
 
 	if (vp->flag&SPECIAL)
 		getspec(vp);
@@ -487,6 +487,13 @@ getint(struct tbl *vp, mksh_ari_t *nump, bool arith)
 	base = 10;
 	num = 0;
 	neg = 0;
+#ifdef MKSH_DISABLE_DEPRECATED
+	if (arith && s[0] == '0' && (s[1] | 0x20) == 'x') {
+		s += 2;
+		base = 16;
+		have_base = true;
+	}
+#else
 	if (arith && *s == '0' && *(s+1)) {
 		s++;
 		if (*s == 'x' || *s == 'X') {
@@ -499,19 +506,20 @@ getint(struct tbl *vp, mksh_ari_t *nump, bool arith)
 			base = 8;
 		have_base = true;
 	}
-	for (c = *s++; c ; c = *s++) {
+#endif
+	while ((c = *s++)) {
 		if (c == '-') {
 			neg++;
 			continue;
 		} else if (c == '#') {
-			base = (int)num;
-			if (have_base || base < 1 || base > 36)
+			if (have_base || num < 1 || num > 36)
 				return (-1);
+			base = (int)num;
 			if (base == 1) {
 				unsigned int wc;
 
 				if (!UTFMODE)
-					wc = *(unsigned char *)s;
+					wc = *(const unsigned char *)s;
 				else if (utf_mbtowc(&wc, s) == (size_t)-1)
 					/* OPTU-8 -> OPTU-16 */
 					/*
@@ -519,7 +527,7 @@ getint(struct tbl *vp, mksh_ari_t *nump, bool arith)
 					 * the same as 1#\x80 does, thus is
 					 * not round-tripping correctly XXX)
 					 */
-					wc = 0xEF00 + *(unsigned char *)s;
+					wc = 0xEF00 + *(const unsigned char *)s;
 				*nump = (mksh_ari_t)wc;
 				return (1);
 			}
@@ -538,9 +546,7 @@ getint(struct tbl *vp, mksh_ari_t *nump, bool arith)
 			return (-1);
 		num = num * base + c;
 	}
-	if (neg)
-		num = -num;
-	*nump = num;
+	*nump = neg ? -((mksh_ari_t)num) : (mksh_ari_t)num;
 	return (base);
 }
 
@@ -556,7 +562,7 @@ setint_v(struct tbl *vq, struct tbl *vp, bool arith)
 
 	if ((base = getint(vp, &num, arith)) == -1)
 		return (NULL);
-	setint_n(vq, num);
+	setint_n(vq, num, 0);
 	if (vq->type == 0)
 		/* default base */
 		vq->type = base;
@@ -565,7 +571,7 @@ setint_v(struct tbl *vq, struct tbl *vp, bool arith)
 
 /* convert variable vq to integer variable, setting its value to num */
 void
-setint_n(struct tbl *vq, mksh_ari_t num)
+setint_n(struct tbl *vq, mksh_ari_t num, int newbase)
 {
 	if (!(vq->flag & INTEGER) && (vq->flag & ALLOC)) {
 		vq->flag &= ~ALLOC;
@@ -573,6 +579,8 @@ setint_n(struct tbl *vq, mksh_ari_t num)
 		afree(vq->val.s, vq->areap);
 	}
 	vq->val.i = num;
+	if (newbase != 0)
+		vq->type = newbase;
 	vq->flag |= ISSET|INTEGER;
 	if (vq->flag&SPECIAL)
 		setspec(vq);
@@ -585,7 +593,7 @@ formatstr(struct tbl *vp, const char *s)
 	char *p, *q;
 	size_t psiz;
 
-	olen = utf_mbswidth(s);
+	olen = (int)utf_mbswidth(s);
 
 	if (vp->flag & (RJUST|LJUST)) {
 		if (!vp->u2.field)
@@ -693,7 +701,7 @@ typeset(const char *var, uint32_t set, uint32_t clr, int field, int base)
 	struct tbl *vpbase, *t;
 	char *tvar;
 	const char *val;
-	int len;
+	size_t len;
 	bool vappend = false;
 
 	/* check for valid variable name, search for value */
@@ -713,11 +721,12 @@ typeset(const char *var, uint32_t set, uint32_t clr, int field, int base)
 		 * IMPORT is only used when the shell starts up and is
 		 * setting up its environment. Allow only simple array
 		 * references at this time since parameter/command
-		 * substitution is preformed on the [expression] which
+		 * substitution is performed on the [expression] which
 		 * would be a major security hole.
 		 */
 		if (set & IMPORT) {
-			int i;
+			size_t i;
+
 			for (i = 1; i < len - 1; i++)
 				if (!ksh_isdigit(val[i]))
 					return (NULL);
@@ -738,9 +747,9 @@ typeset(const char *var, uint32_t set, uint32_t clr, int field, int base)
 		val = NULL;
 		/* handle foo[*] => foo (whole array) mapping for R39b */
 		len = strlen(tvar);
-		if (len > 3 && tvar[len-3] == '[' && tvar[len-2] == '*' &&
-		    tvar[len-1] == ']')
-			tvar[len-3] = '\0';
+		if (len > 3 && tvar[len - 3] == '[' && tvar[len - 2] == '*' &&
+		    tvar[len - 1] == ']')
+			tvar[len - 3] = '\0';
 	}
 
 	if (set_refflag == SRF_ENABLE) {
@@ -944,7 +953,7 @@ unset(struct tbl *vp, int flags)
 const char *
 skip_varname(const char *s, int aok)
 {
-	int alen;
+	size_t alen;
 
 	if (s && ksh_isalphx(*s)) {
 		while (*++s && ksh_isalnux(*s))
@@ -1132,7 +1141,7 @@ getspec(struct tbl *vp)
 		return;
 	}
 	vp->flag &= ~SPECIAL;
-	setint_n(vp, i);
+	setint_n(vp, i, 0);
 	vp->flag |= SPECIAL;
 }
 
@@ -1169,6 +1178,7 @@ setspec(struct tbl *vp)
 			struct stat statb;
 
 			s = str_val(vp);
+			/* LINTED use of access */
 			if (s[0] == '/' && access(s, W_OK|X_OK) == 0 &&
 			    stat(s, &statb) == 0 && S_ISDIR(statb.st_mode))
 				strdupx(tmpdir, s, APERM);
@@ -1336,11 +1346,11 @@ arraysearch(struct tbl *vp, uint32_t val)
  * to point to the open bracket. Returns 0 if there is no matching
  * closing bracket.
  */
-int
+size_t
 array_ref_len(const char *cp)
 {
 	const char *s = cp;
-	int c;
+	char c;
 	int depth = 0;
 
 	while ((c = *s++) && (c != ']' || --depth))
@@ -1374,40 +1384,36 @@ mksh_uari_t
 set_array(const char *var, bool reset, const char **vals)
 {
 	struct tbl *vp, *vq;
-	mksh_uari_t i, j = 0;
-	const char *ccp;
-#ifndef MKSH_SMALL
+	mksh_uari_t i = 0, j = 0;
+	const char *ccp = var;
 	char *cp = NULL;
-#endif
+	size_t n;
 
-	/* to get local array, use "typeset foo; set -A foo" */
-#ifndef MKSH_SMALL
-	i = strlen(var);
-	if (i > 0 && var[i - 1] == '+') {
+	/* to get local array, use "local foo; set -A foo" */
+	n = strlen(var);
+	if (n > 0 && var[n - 1] == '+') {
 		/* append mode */
 		reset = false;
-		strndupx(cp, var, i - 1, ATEMP);
+		strndupx(cp, var, n - 1, ATEMP);
+		ccp = cp;
 	}
-#define CPORVAR	(cp ? cp : var)
-#else
-#define CPORVAR	var
-#endif
-	vp = global(CPORVAR);
+	vp = global(ccp);
 
 	/* Note: AT&T ksh allows set -A but not set +A of a read-only var */
 	if ((vp->flag&RDONLY))
-		errorfx(2, "%s: %s", CPORVAR, "is read only");
+		errorfx(2, "%s: %s", ccp, "is read only");
 	/* This code is quite non-optimal */
-	if (reset)
+	if (reset) {
 		/* trash existing values and attributes */
 		unset(vp, 1);
+		/* allocate-by-access the [0] element to keep in scope */
+		arraysearch(vp, 0);
+	}
 	/*
 	 * TODO: would be nice for assignment to completely succeed or
 	 * completely fail. Only really effects integer arrays:
 	 * evaluation of some of vals[] may fail...
 	 */
-	i = 0;
-#ifndef MKSH_SMALL
 	if (cp != NULL) {
 		/* find out where to set when appending */
 		for (vq = vp; vq; vq = vq->u.array) {
@@ -1418,9 +1424,7 @@ set_array(const char *var, bool reset, const char **vals)
 		}
 		afree(cp, ATEMP);
 	}
-#endif
 	while ((ccp = vals[i])) {
-#ifndef MKSH_SMALL
 		if (*ccp == '[') {
 			int level = 0;
 
@@ -1441,7 +1445,6 @@ set_array(const char *var, bool reset, const char **vals)
 			} else
 				ccp = vals[i];
 		}
-#endif
 
 		vq = arraysearch(vp, j);
 		/* would be nice to deal with errors here... (see above) */
